"""
Content type defines the structure or schema of a page or a section of your web
or mobile property. To create content for your application, you are required
to first create a content type, and then create entries using the
content type.
"""

# ************* Module asset **************
# Your code has been rated at 10.00/10 by pylint

from urllib import parse

from contentstack.entry import Entry
from contentstack.query import Query


class ContentType:
    """
    Content type defines the structure or schema of a page or a
    section of your web or mobile property. To create
    content for your application, you are required to
    first create a content type, and then create entries using the
    content type.
    """

    def __init__(self, http_instance, content_type_uid):
        self.http_instance = http_instance
        self.__content_type_uid = content_type_uid
        self.local_param = {}

    def entry(self, entry_uid: str):
        r"""
        An entry is the actual piece of content created using one of the defined content types.
        :param entry_uid: {str} -- unique ID of the entry that you wish to fetch
        :return: Entry -- Returns the Entry class object so we can chain the entry functions

        --------------------------------

        [Example:]
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> entry = content_type.entry(uid='entry_uid')
        --------------------------------
        """
        if self.__content_type_uid is None:
            raise PermissionError('Please provide valid content_type_uid')
        if entry_uid is None:
            raise PermissionError('Please provide valid entry uid')
        entry = Entry(self.http_instance, self.__content_type_uid, entry_uid=entry_uid)
        return entry

    def query(self):
        """
        It returns query class object so we can query on entry of specified ContentType
        :return: Query -- query object instance, so we can chain the query functions to it.
        ------------------------------
        [Example:]

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> query = content_type.query()
        ------------------------------
        """
        if self.__content_type_uid is None:
            raise PermissionError('Kindly provide content_type_uid')
        return Query(self.http_instance, self.__content_type_uid)

    def fetch(self):
        """
        This method is useful to fetch ContentType of the of the stack.
        :return:dict -- contentType response
        ------------------------------
        Example:

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> some_dict = {'abc':'something'}
            >>> response = content_type.fetch(some_dict)
        ------------------------------
        """
        if self.__content_type_uid is None:
            raise KeyError('content_type_uid can not be None to fetch contenttype')
        self.local_param['environment'] = self.http_instance.headers['environment']
        uri = '{}/content_types/{}'.format(self.http_instance.endpoint, self.__content_type_uid)
        encoded_params = parse.urlencode(self.local_param)
        url = '{}?{}'.format(uri, encoded_params)
        result = self.http_instance.get(url)
        return result

    def find(self, params=None):
        """
        This method is useful to fetch ContentType of the of the stack.
        :param params: dictionary of params
        :return:dict -- contenttype response
        ------------------------------
        Example:

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type()
            >>> some_dict = {'abc':'something'}
            >>> response = content_type.find(param=some_dict)
        ------------------------------
        """
        self.local_param['environment'] = self.http_instance.headers['environment']
        if params is not None:
            self.local_param.update(params)
        encoded_params = parse.urlencode(self.local_param)
        url = '{}?{}'.format('{}/content_types'.format(self.http_instance.endpoint), encoded_params)
        result = self.http_instance.get(url)
        return result



"""
The Image Delivery API is used to retrieve, manipulate and/or convert image
files of your Contentstack account and deliver it to your web or mobile properties.
It is an second parameter in which we want to place different manipulation key and
value in array form ImageTransform method is define for image manipulation with
different transform_params in second parameter in array form
"""


# ************* Module image_transform **************
# Your code has been rated at 10.00/10  by pylint


class ImageTransform:  # pylint: disable=too-few-public-methods
    """
    The Image Delivery API is used to retrieve, manipulate and/or convert image
    files
    """

    def __init__(self, http_instance, image_url, **kwargs):
        """
        creates instance of the ImageTransform class
        :param httpInstance: instance of HttpsConnection
        :param image_url: url on which manipulation required, Image transformation url
        :param kwargs: parameter in which we want to place different
        manipulation key-worded, variable-length argument list
        ------------------------------
        Example::
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> stack.image_transform('image_url', width=100, height=100)
        ------------------------------
        """
        super().__init__()
        self.http_instance = http_instance
        self.image_url = image_url
        self.image_params = kwargs

    def get_url(self):
        """
        Returns a complete url after concatenate with parameters
        :return: updated asset url
        ------------------------------
        Example::
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> image_url = stack.image_transform('image_url', width=100, height=100)
            >>> result = image_url.fetch()
        ------------------------------
        """
        args = ['{0}={1}'.format(k, v) for k, v in self.image_params.items()]
        if args:
            self.image_url += '?{0}'.format('&'.join(args))
        return self.image_url



"""
Missing docstring
Common Query for Entry and Assets
"""
import enum


# ************* Module basequery.py **************
# Your code has been rated at 10.00/10

class QueryOperation(enum.Enum):
    """
    QueryOperation is enum that Provides Options to perform operation to query the result.

    Available Options for QueryOperation are below.
    EQUALS, NOT_EQUALS, INCLUDES, EXCLUDES, IS_LESS_THAN, IS_LESS_THAN_OR_EQUAL
    IS_GREATER_THAN, IS_GREATER_THAN_OR_EQUAL, EXISTS, MATCHES

    Arguments:
        enum {QueryOperation} -- Type of operation to perform
    """
    EQUALS = ""
    NOT_EQUALS = '$ne'
    INCLUDES = '$in'
    EXCLUDES = '$nin'
    IS_LESS_THAN = '$lt'
    IS_LESS_THAN_OR_EQUAL = '$lte'
    IS_GREATER_THAN = '$gt'
    IS_GREATER_THAN_OR_EQUAL = '$gte'
    EXISTS = '$exists'
    MATCHES = '$regex'


def _get_operation_value(fields):
    value = fields[0] if isinstance(fields, list) and len(fields) == 1 else fields
    return value


class BaseQuery:
    """
    Common Query class works for Query As well as Asset
    """

    def __init__(self):
        self.parameters = {}
        self.query_params = {}

    def where(self, field_uid: str, query_operation: QueryOperation, fields=None):
        """
        Get entries containing the field values matching the condition in the query.
        Arguments:
            field_uid {str} -- [accept field uid for the operation]
            query_operation {QueryOperation} -- Type of operation to perform
            fields {list} - list of string
        """
        if None not in (field_uid, query_operation):
            result = _get_operation_value(fields) if query_operation.name == "EQUALS" \
                else {query_operation.value: fields}
            self.parameters[field_uid] = result
        return self

    def include_count(self):
        """Retrieve count and data of objects in result
        Returns:
            [Query] -- Query object, so you can chain this call.
        ==============================
        [Example:]
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'access_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> query = content_type.query()
            >>> query = query.include_count()
            >>> result = query.find()
        ==============================
        """
        self.query_params["include_count"] = 'true'
        return self

    def skip(self, skip_count: int):
        """
        The number of objects to skip before returning any.
        skip_count No of objects to skip from returned objects
        :param skip_count:
        :return: self
        """
        self.query_params["skip"] = str(skip_count)
        return self

    def limit(self, limit_count: int):
        """
        A limit on the number of objects to return.
        :param limit_count:
        :return: self
        """
        self.query_params["skip"] = str(limit_count)
        return self

    def order_by_ascending(self, key: str):
        """
        you can sort them in the ascending order with respect to the
        value of a specific field in the response body.

        :param key:  key on which ascending order to be implemented
        :return: self
        """
        self.query_params["asc"] = str(key)
        return self

    def order_by_descending(self, key: str):
        """
        you can sort them in the descending order with respect to the value
        of a specific field in the response body.
        :param key:  key on which descending order to be implemented
        :return: self - Class instance, So that method chaining can be performed
        """
        self.query_params["desc"] = str(key)
        return self

    def param(self, key: str, value):
        """
        Adds Parameters to the to the request

        Arguments:
            key {str} -- Key of the parameter
            value {[any]} -- Value of the parameter

        Raises:
            KeyError: When None found in key or value

        Returns:
            [self] -- instance of the class

        -----------------------------------

        [Example]
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'access_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> query = content_type.query()
            >>> query = query.param("key", "value")
            >>> result = query.find()

        -----------------------------------
        """
        if None in (key, value):
            raise KeyError('Invalid key or value')
        self.query_params[key] = str(value)
        return self

    def add_params(self, param: dict):
        """
        Adds Parameters to the to the request
        Arguments:
            param {dict} --  parameters
        Returns:
            [self] -- Class instance, So that method chaining can be performed
        """
        self.query_params.update(param)
        return self

    def query(self, key: str, value):
        """
        Adds key value pairs to the to the query parameters
        Arguments:
            key {str} -- key of the query param
            value {any} -- value of query param
        Raises:
            KeyError: when key or value found None
        Returns:
            self-- Class instance, So that method chaining can be performed
        """
        if None in (key, value):
            raise KeyError('Invalid key or value')
        self.parameters[key] = str(value)
        return self

    def remove_param(self, key: str):
        """
        Remove provided query key from custom query if exist.
        :param key {str} -- The key to be constrained
        :return: self -- So that method chaining can be performed

        ----------------------------------
        [Example]:
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> query = content_type.query()
            >>> query = query.remove_query("query_key")
            >>> result = query.find()
        ----------------------------------
        """
        if key is None:
            raise ValueError('Kindly provide valid key')
        if key in self.query_params:
            self.query_params.pop(key, None)
        return self



r"""Assets refer to all the media files (images, videos, PDFs, audio files,
and so on) uploaded in your Contentstack repository for future use.

These files can be attached and used in multiple entries.
"""

# ************* Module asset **************
# Your code has been rated at 10/10 by pylint

from urllib import parse


class Asset:
    r"""Asset refer to all the media files (images, videos, PDFs, audio files, and so on)."""

    def __init__(self, http_instance, uid=None):
        self.http_instance = http_instance
        self.__query_params = {}
        self.__uid = uid
        if self.__uid is None or self.__uid.strip() == 0:
            raise KeyError('Please provide valid uid')
        self.base_url = '{}/assets/{}'.format(self.http_instance.endpoint, self.__uid)
        if 'environment' in self.http_instance.headers:
            self.__query_params['environment'] = self.http_instance.headers['environment']
            # self.http_instance.headers.pop('environment')

    def environment(self, environment):
        r"""Provide the name of the environment if you wish to retrieve the assets published
        in a particular environment.

        :param environment {str} - name of the environment

        :return: Asset, so we can chain the call

        -------------------------------
        [Example]:

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> asset = stack.asset(uid='asset_uid')
            >>> asset = asset.environment('production')
        -------------------------------
        """
        if environment is not None or environment is str:
            self.__query_params['environment'] = environment
        return self

    def remove_environment(self):
        r"""Removes environment from the request params

        :return: Asset, so we can chain the call

        -------------------------------
        [Example]:

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> asset = stack.asset(uid='asset_uid')
            >>> asset = asset.remove_header()
            >>> asset.fetch()
        -------------------------------
        """
        if 'environment' in self.http_instance.headers:
            self.http_instance.headers.pop('environment')
        return self

    def params(self, key, value):
        r"""params is used to pass additional parameters to the asset query

        :param key: key of the query parameter

        :param value: value of the query parameter

        :return: Asset, so we can chain the call

        -----------------------------
        Example::

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> asset = stack.asset(uid='asset_uid')
            >>> asset = asset.params('key', 'value')
        -----------------------------
        """
        if None in (key, value) or not isinstance(key, str):
            raise KeyError('Kindly provide valid params')
        self.__query_params[key] = value
        return self

    def relative_urls(self):
        r"""Include the relative URLs of the assets in the response.

        :return: Asset, so we can chain the call

        ----------------------------
        Example::

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> asset = stack.asset(uid='asset_uid')
            >>> asset = asset.relative_urls()
        ----------------------------
        """
        self.__query_params['relative_urls'] = 'true'
        return self

    def include_dimension(self):
        r"""Include the dimensions (height and width) of the image in the response.
        Supported image types: JPG, GIF, PNG, WebP, BMP, TIFF, SVG, and PSD.

        :return: Asset, so we can chain the call

        ----------------------------
        Example::

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> asset = stack.asset(uid='asset_uid')
            >>> asset = asset.include_dimension()
        ----------------------------
        """
        self.__query_params['include_dimension'] = "true"
        return self

    def fetch(self):
        r"""This call fetches the latest version of a specific asset of a particular stack.

        :return: json response of asset

        -----------------------------
        [Example]:

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> asset = stack.asset(uid='asset_uid')
            >>> result = asset.fetch()
        ------------------------------
        """
        url = '{}?{}'.format(self.base_url, parse.urlencode(self.__query_params))
        return self.http_instance.get(url)



"""
Utils
contentstack
Last modified by Shailesh Mishra on 06/08/20.
Copyright 2019 Contentstack. All rights reserved.
"""

# ************* Module utility checked using pylint **************
# Your code has been rated at 10.00/10 (previous run: 10.00/10, +0.00)


import json
import logging
from urllib import parse


def config_logging(logging_type: logging.WARNING):
    """
    This is to create logging config
    :param logging_type:  Level of the logging
    :return: basicConfig instance
    """
    logging.basicConfig(
        filename='app.log',
        level=logging_type,
        format='[%(asctime)s] {%(pathname)s:%(lineno)d} %(levelname)s - %(message)s',
        datefmt='%H:%M:%S'
    )


class Utils:
    """
    Utility for the contentstack
    """

    @staticmethod
    def config_logging():
        """ Setting up logging """
        logging.basicConfig(
            filename='report_log.log',
            format='%(asctime)s - %(message)s',
            level=logging.INFO
        )

    @staticmethod
    def setup_logger():
        """setup logger for the application"""
        return logging.getLogger("Config")

    @staticmethod
    def log(message):
        """this generates log message"""
        logging.debug(message)

    @staticmethod
    def do_url_encode(params):
        """
        To encode url with query parameters
        :param params:
        :return: encoded url
        """
        return parse.urlencode(params)

    @staticmethod
    def get_complete_url(base_url: str, params: dict):
        """ creates complete url using base_url and their respective parameters
        :param base_url:
        :param params:
        :return:
        """
        if 'query' in params:
            params["query"] = json.dumps(params["query"])
        query = parse.urlencode(params)
        return '{}&{}'.format(base_url, query)



r"""This call fetches the list of all the assets of a particular stack.
It also returns the content of each asset in JSON format.
You can also specify the environment of which you wish to get the assets.
"""

# ************* Module assetquery **************
# Your code has been rated at 10/10 by pylint

import json
from contentstack.utility import Utils
from contentstack.basequery import BaseQuery


class AssetQuery(BaseQuery):
    """
    This call fetches the list of all the assets of a particular stack.
    """

    def __init__(self, http_instance):
        super().__init__()
        self.http_instance = http_instance
        self.__query_params = {}
        self.base_url = "{}/assets".format(self.http_instance.endpoint)
        if "environment" in self.http_instance.headers:
            env = self.http_instance.headers["environment"]
            self.base_url = "{}?{}".format(self.base_url, "environment={}".format(env))

    def environment(self, environment):
        r"""Provide the name of the environment if you wish to retrieve the assets published
        in a particular environment.

        :param environment: environment of the stack

        :return: AssetQuery - so we can chain the call

        -----------------------------
        [Example]:

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> result = stack.asset_query().environment('production').find()
        ------------------------------
        """
        self.__query_params["environment"] = environment
        return self

    def version(self, version):
        r"""Specify the version number of the asset that you wish to retrieve.
        If the version is not specified, the details of the latest version will be retrieved.
        To retrieve a specific version, keep the environment parameter blank.

        :param version: version number of the asset that you wish to retrieve

        :return: AssetQuery: so we can chain the call

        -----------------------------
        [Example]:

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> result = stack.asset_query().version(3).find()
        ------------------------------
        """
        self.__query_params["version"] = version
        return self

    def include_dimension(self):
        r"""Include the dimensions (height and width) of the image in the response.
        Supported image types: JPG, GIF, PNG, WebP, BMP, TIFF, SVG, and PSD

        :return: AssetQuery: so we can chain the call

        -----------------------------
        [Example]:

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> result = stack.asset_query().include_dimension().find()
        ------------------------------
        """
        self.__query_params["include_dimension"] = "true"
        return self

    def relative_url(self):
        r"""include the relative URLs of the assets in the response.

        :return: AssetQuery: so we can chain the call

        -----------------------------
        [Example]:

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> result = stack.asset_query().relative_url().find()
        ------------------------------
        """
        self.__query_params["relative_urls"] = "true"
        return self

    def find(self):
        r"""This call fetches the list of all the assets of a particular stack.
        It also returns the content of each asset in JSON format.
        Learn more about Assets
        [https://www.contentstack.com/docs/content-managers/work-with-assets].

        :return: json result, List of asset object

        -----------------------------
        [Example]:

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> result = stack.asset_query().find()

        """
        if self.parameters is not None and len(self.parameters) > 0:
            self.__query_params["query"] = json.dumps(self.parameters)
        url = Utils.get_complete_url(self.base_url, self.__query_params)
        return self.http_instance.get(url)



"""
This module implements the Requests API.
"""

# ************* Module https_connection.py **************
# Your code has been rated at 10.00/10  by pylint

import platform
from json import JSONDecodeError

import requests
from requests.exceptions import Timeout, HTTPError

import contentstack


def get_os_platform():
    """ returns client platform """
    os_platform = platform.system()
    if os_platform == 'Darwin':
        os_platform = 'macOS'
    elif not os_platform or os_platform == 'Java':
        os_platform = None
    elif os_platform and os_platform not in ['macOS', 'Windows']:
        os_platform = 'Linux'
    os_platform = {'name': os_platform, 'version': platform.release()}
    return os_platform


def user_agents():
    """User Agents for the Https"""
    header = {'sdk': dict(name=contentstack.__package__, version=contentstack.__version__),
              'os': get_os_platform,
              'Content-Type': 'application/json'}
    package = "contentstack-python/{}".format(contentstack.__version__)
    return {'User-Agent': str(header), "X-User-Agent": package}


class HTTPSConnection:  # R0903: Too few public methods
    """Make Https Request to fetch the result as per requested url"""

    def __init__(self, endpoint, headers):
        if None not in (endpoint, headers):
            self.payload = None
            self.endpoint = endpoint
            self.headers = headers
            self.default_timeout = 10

    def get(self, url):
        """
        Here we create a response object ‘response’ which will store the request-response.
        We use requests.get method since we are sending a GET request.
        The four arguments we pass are url, verify(ssl), timeout, headers
        """
        try:
            self.headers.update(user_agents())
            response = requests.get(url, verify=True, headers=self.headers)
            response.encoding = 'utf-8'
            return response.json()
        except Timeout:
            raise TimeoutError('The request timed out')
        except ConnectionError:
            raise ConnectionError('Connection error occurred')
        except JSONDecodeError:
            raise TypeError('Invalid JSON in request')
        except HTTPError:
            raise HTTPError('Http Error Occurred')

    def update_connection_timeout(self, timeout: int):
        """Facilitate to update timeout for the https request"""
        self.default_timeout = timeout



"""
The Get a single entry request fetches a particular entry of a content type.
API Reference: https://www.contentstack.com/docs/developers/apis/content-delivery-api/#single-entry
"""

from urllib import parse

from contentstack.entryqueryable import EntryQueryable


# ************* Module stack **************
# Your code has been rated at 9.89/10 by pylint


class Entry(EntryQueryable):
    """
    An entry is the actual piece of content that you want to publish.
    Entries can be created for one of the available content types.

    Entry works with
    version={version_number}
    environment={environment_name}
    locale={locale_code
    """

    def __init__(self, http_instance, content_type_uid, entry_uid):
        super().__init__()
        EntryQueryable.__init__(self)
        self.entry_param = {}
        self.http_instance = http_instance
        self.content_type_id = content_type_uid
        self.entry_uid = entry_uid
        self.base_url = self.__get_base_url()

    def environment(self, environment):
        """
        Enter the name of the environment of which the entries needs to be included
        Example: production
        :param environment: {str} name of the environment of which the entries needs to be included.
        :return: Entry, so you can chain this call.
        ------------------------------
        Example::

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> entry = content_type.entry(uid='entry_uid')
            >>> entry.environment('production')
            >>> result = entry.fetch()
        ------------------------------
        """
        if environment is None:
            raise KeyError('Kindly provide a valid environment')
        self.http_instance.headers['environment'] = environment
        return self

    def version(self, version):
        """When no version is specified, it returns the latest version
        To retrieve a specific version, specify the version number under this parameter.
        In such a case, DO NOT specify any environment. Example: 4

        :param version: {int} -- version
        :return: Entry, so you can chain this call.
        ------------------------------
        Example::

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> entry = content_type.entry(uid='entry_uid')
            >>> entry.version(4)
            >>> result = entry.fetch()
        ------------------------------
        """
        if version is None:
            raise KeyError('Kindly provide a valid version')
        self.entry_param['version'] = version
        return self

    def param(self, key: str, value: any):
        """
        This method is useful to add additional Query parameters to the entry
        :param key: {str} -- key The key as string which needs to be added to an Entry
        :param value: {object} -- value The value as string which needs to be added to an Entry
        :return: Entry, so you can chain this call.
        -----------------------------
        Example::

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> entry = content_type.entry(uid='entry_uid')
            >>> entry = entry.param('key', 'value')
            >>> result = entry.fetch()
        -----------------------------
        """
        if None in (key, value) and not isinstance(key, str):
            raise ValueError('Kindly provide valid key and value arguments')
        self.entry_param[key] = value
        return self

    def __get_base_url(self):
        if None in (self.http_instance, self.content_type_id, self.entry_uid):
            raise KeyError('Provide valid http_instance, content_type_uid or entry_uid')
        url = '{}/content_types/{}/entries/{}' \
            .format(self.http_instance.endpoint, self.content_type_id, self.entry_uid)
        return url

    def fetch(self):
        """
        Fetches the latest version of the entries from stack
        :return: Entry, so you can chain this call.
        -------------------------------
        [Example:]

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> entry = content_type.entry('uid')
            >>> result = entry.fetch()
        -------------------------------
        """

        if 'environment' in self.http_instance.headers:
            self.entry_param['environment'] = self.http_instance.headers['environment']
        if len(self.entry_queryable_param) > 0:
            self.entry_param.update(self.entry_queryable_param)
        encoded_string = parse.urlencode(self.entry_param, doseq=True)
        url = '{}?{}'.format(self.base_url, encoded_string)
        return self.http_instance.get(url)



"""
Contentstack provides certain queries that you can use to fetch filtered results
"""
import json
import enum
from urllib import parse
from contentstack.basequery import BaseQuery
from contentstack.entryqueryable import EntryQueryable


# Your code has been rated at 10.00/10 (previous run: 10.00/10, +0.00)


class QueryType(enum.Enum):
    """
    Get entries that satisfy all the conditions provided by enum(AND, OR)
    enum ([AND, OR]): Get entries that satisfy all the conditions provided by enum
    AND: Get entries that satisfy all the conditions provided in the '$and' query
    OR: Get entries that satisfy all the conditions provided in the '$or' query
    """
    AND = "$and"
    OR = '$or'


class Query(BaseQuery, EntryQueryable):
    """
    Contentstack provides certain queries that you can use to fetch filtered results.
    You can use queries for Entries API requests.
    [API Reference]:https://www.contentstack.com/docs/developers/apis/content-delivery-api/#queries]

    ---------------------------------------
    Example:
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> query = stack.content_type('content_type_uid').query()
            >>> result = query.locale('locale-code').excepts('field_uid').limit(4).skip(5).find()
    """

    def __init__(self, http_instance, content_type_uid):
        super().__init__()
        # BaseQuery.__init__(self)
        EntryQueryable.__init__(self)
        self.content_type_uid = content_type_uid
        self.http_instance = http_instance
        if self.content_type_uid is None:
            raise PermissionError('You are not allowed here without content_type_uid')
        self.base_url = '{}/content_types/{}/entries' \
            .format(self.http_instance.endpoint, self.content_type_uid)

    def query_operator(self, query_type: QueryType, *query_objects):
        """
        Get entries that satisfy all the conditions provided in the '$and' query.
        Arguments:
            query_objects {Query} -- query_objects for variable number
            of arguments of type Query Object.
        Raises:
            ValueError: If query_objects is None
        Returns:
            Query -- Query object, so you can chain this call.
        ---------------------------------
        [Example]:
            >>> import contentstack
            >>> from contentstack.basequery import QueryOperation
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> query = stack.content_type('content_type1').query()
            >>> self.query1 = stack.content_type('content_type2').query()
            >>> self.query2 = stack.content_type('content_type3').query()
            >>> query1 = self.query1.where("price", QueryOperation.IS_LESS_THAN, fields=90)
            >>> query2 = self.query2.where("discount", QueryOperation.INCLUDES, fields=[20, 45])
            >>> query = query.query_operator(query1, query2)
            >>> result = query.find()
        ---------------------------------
        """
        __container = []
        if len(query_objects) > 0:
            for query in query_objects:
                __container.append(query.parameters)
        if len(self.parameters) > 0:
            self.parameters.clear()
        self.query_params["query"] = json.dumps({query_type.value: __container})
        return self

    # def and_query(self, *query_objects):
    #     """
    #     Get entries that satisfy all the conditions provided in the '$and' query.
    #     Arguments:
    #         query_objects {Query} -- query_objects for variable number
    #         of arguments of type Query Object.
    #     Raises:
    #         ValueError: If query_objects is None
    #     Returns:
    #         Query -- Query object, so you can chain this call.
    #     ---------------------------------
    #     [Example]:
    #         >>> import contentstack
    #         >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
    #         >>> query = stack.content_type('content_type_uid').query()
    #         >>> query_one = query.where('field_uid',
    #                                 QueryOperation.EQUALS, fields=['field1', 'field2', 'field3'])
    #         >>> query_two = query.where('field_uid',
    #                                 QueryOperation.EQUALS, fields=['field1', 'field2', 'field3'])
    #         >>> result = query.and_query(query_one, query_two).find()
    #     ---------------------------------
    #     """
    #     __container = []
    #     if len(query_objects) > 0:
    #         for query in query_objects:
    #             __container.append(query.parameters)
    #     self.query_params["query"] = json.dumps({"$and": __container})
    #     if len(self.parameters) > 0:
    #         self.parameters.clear()
    #     return self
    #
    # def or_query(self, *query_objects):
    #     """
    #     Get all entries that satisfy at least
    #     one of the given conditions provided in the '$or' query.
    #     Arguments:
    #         query_objects {object} -- query_objects for variable
    #         number of arguments of type Query Object.
    #     Raises:
    #         ValueError: If query_objects is None
    #     Returns:
    #         Query -- Query object, so you can chain this call.
    #     ----------------------------------
    #     [Example]:
    #
    #         >>> import contentstack
    #         >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
    #         >>> query = stack.content_type('content_type_uid').query()
    #         >>> query_one = query.where('field_uid',
    #                             QueryOperation.EQUALS, fields=['field1', 'field2', 'field3'])
    #         >>> query_two = query.where('field_uid',
    #                             QueryOperation.EQUALS, fields=['field1', 'field2', 'field3'])
    #         >>> result = query.or_query(query_one, query_two).find()
    #     ----------------------------------
    #     """
    #     __container = []
    #     if len(query_objects) > 0:
    #         for i in range(len(query_objects)):
    #             obj = query_objects[i].parameters[list(query_objects[i].parameters)[i]]
    #             __container.append(obj)
    #     self.query_params["query"] = json.dumps({"$or": __container})
    #     if len(self.parameters) > 0:
    #         self.parameters.clear()
    #     return self

    def tags(self, *tags):
        """
        Include tags with which to search entries accepts variable-length argument lists
        Arguments:
            tags {list of str} -- tags accepts variable-length argument lists to search entries.
        Returns:
            [Query] -- Query object, so you can chain this call.

        ----------------------------------
        Example:
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> query = content_type.query()
            >>> query = query.tags('black', 'gold', 'silver')
            >>> result = query.find()
        ----------------------------------
        """
        if tags is not None:
            self.query_params["tags"] = ",".join(tags)
        return self

    def search(self, value: str):
        """
        This method provides only the entries matching
        the specified value.
        Arguments:
            value {str} -- value used to match or compare
        Raises:
            ValueError: If value is None
            ValueError: If type od value is not str
        Returns:
            [Query] -- Query object, so you can chain this call.
        -------------------------------------
        [Example]
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> query = content_type.query()
            >>> query = query.search("search_keyword")
            >>> result = query.find()
        -------------------------------------
        """
        if value is not None:
            self.query_params["typeahead"] = value
        return self

    def where_in(self, key: str, query_object):
        """Get entries having values based on referenced fields.
        This query retrieves all entries that satisfy the query
        conditions made on referenced fields.
        Arguments:
            key {str} -- The key to be constrained
        Raises:
            ValueError: If key is None
            ValueError: If key is not str type
        Returns:
            [Query] -- Query object, so you can chain this call.
        -------------------------------------
        [Example]
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> query = content_type.query()
            >>> query = query.where_in("brand")
            >>> result = query.find()
        -------------------------------------
        :param key:
        :type query_object: object
        """
        if isinstance(key, str) and query_object is not None and isinstance(query_object, Query):
            self.query_params["query"] = {key: {"$in_query": query_object.parameters}}
        else:
            raise ValueError('Invalid Key or Value provided')
        return self

    def where_not_in(self, key, query_object):
        """Get entries having values based on referenced fields.
        This query works the opposite of $in_query and
        retrieves all entries that does not satisfy query
        conditions made on referenced fields.
        Arguments:
            key {str} -- The key to be constrained
        Raises:
            ValueError: If key is None
            ValueError: If key is not str type
        Returns:
            [Query] -- Query object, so you can chain this call.
        -------------------------------------
        [Example]
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> query = content_type.query()
            >>> query = query.where_not_in("brand")
            >>> result = query.find()
        -------------------------------------
        """
        # pylint: disable=W0212
        if isinstance(key, str) and query_object is not None and isinstance(query_object, Query):
            self.query_params["query"] = {key: {"$nin_query": query_object.parameters}}
        else:
            raise ValueError('Invalid Key or Value provided')
        return self

    def find(self):
        """It fetches the query result.
        List of :class:`Entry <contentstack.entry.Entry>` objects.
        Raises:
            ValueError: If content_type_id is None
            ValueError: If content_type_id is empty or not str type
        Returns:
            list[Entry] -- List of <contentstack.entry.Entry>
        -------------------------------------
        [Example]:
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> query = content_type.query()
            >>> result = query.find()
        -------------------------------------
        """
        # if len(self.entry_queryable_param) > 0:
        #     self.query_params.update(self.entry_queryable_param)
        return self.__execute_network_call()

    def find_one(self):
        """It returns only one result.
        Returns:
            list[Entry] -- List of <contentstack.entry.Entry>
        -------------------------------------
        [Example]:
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> query = content_type.query()
            >>> result = query.find_one()
        -------------------------------------
        """
        self.query_params["limit"] = 1
        return self.__execute_network_call()

    def __execute_network_call(self):
        if len(self.entry_queryable_param) > 0:
            self.query_params.update(self.entry_queryable_param)
        if len(self.parameters) > 0:
            self.query_params["query"] = json.dumps(self.parameters)
        if 'environment' in self.http_instance.headers:
            self.query_params['environment'] = self.http_instance.headers['environment']
        encoded_string = parse.urlencode(self.query_params, doseq=True)
        url = '{}?{}'.format(self.base_url, encoded_string)
        return self.http_instance.get(url)



"""
Class that wraps the credentials of the authenticated user. Think of
this as a container that holds authentication related data.
"""

# ************* Module stack **************
# Your code has been rated at 10.00/10

import logging
import enum
from urllib import parse
from contentstack.asset import Asset
from contentstack.assetquery import AssetQuery
from contentstack.contenttype import ContentType
from contentstack.https_connection import HTTPSConnection
from contentstack.image_transform import ImageTransform


class ContentstackRegion(enum.Enum):
    """
    Sets region for the contentstack
    """
    US = 'us'
    EU = 'eu'


class Stack:
    """
    A stack can be defined as a pool of data or a container that holds all
    the content/assets related to a site. It is a collaboration space where multiple users can work
    together to create, edit, approve, and publish content.
    (API Reference)[https://www.contentstack.com/docs/developers/apis/content-delivery-api/#stack]:
    """

    # pylint: disable=too-many-instance-attributes
    # pylint: disable=too-many-arguments
    def __init__(self, api_key, delivery_token, environment,
                 host='cdn.contentstack.io',
                 version='v3', region=ContentstackRegion.US):
        """
        Class that wraps the credentials of the authenticated user. Think of
        this as a container that holds authentication related data.
        :param api_key: api_key of the stack
        :param delivery_token: delivery_token of the stack
        :param environment: environment of the stack
        :param host: (optional) host of the stack default is cdm.contentstack.io
        :param version: (optional) apiVersion of the stack default is v3
        :param region: (optional) region support of the stack default is ContentstackRegion.US
        """
        logging.basicConfig(level=logging.DEBUG)
        self.headers = {}
        self.__query_params = {}
        self.sync_param = {}
        self.endpoint = None
        self.http_instance = None
        self.api_key = api_key
        self.delivery_token = delivery_token
        self.environment = environment
        self.host = host
        self.version = version
        self.region = region
        self.__validate_stack()

    def __validate_stack(self):
        if self.api_key is None or self.api_key == '':
            raise PermissionError('You are not permitted to the stack without valid Api Key')
        if self.delivery_token is None or self.delivery_token == "":
            raise PermissionError('You are not permitted to the stack without valid Delivery Token')
        if self.environment is None or self.environment == "":
            raise PermissionError('You are not permitted to the stack without valid Environment')
        # prepare endpoint for the url:
        if self.region.value != 'us' and self.host == 'cdn.contentstack.io':
            self.host = 'eu-cdn.contentstack.com'
        self.endpoint = 'https://{}/{}'.format(self.host, self.version)
        # prepare Headers:`
        self.headers = {'api_key': self.api_key, 'access_token': self.delivery_token,
                        'environment': self.environment}
        self.http_instance = HTTPSConnection(endpoint=self.endpoint, headers=self.headers)
        # call httpRequest instance & pass the endpoint and headers

    @property
    def get_api_key(self):
        """
        :return: api_key of the stack
        """
        return self.api_key

    @property
    def get_delivery_token(self):
        """
        :return: delivery_token of the stack
        """
        return self.delivery_token

    @property
    def get_environment(self):
        """
        :return: environment of the stack
        """
        return self.environment

    @property
    def get_headers(self):
        """
        :return: validating credentials http header of the stack
        """
        return self.headers

    def content_type(self, content_type_uid=None):
        """
        Content type defines the structure or schema of a page or a section
        of your web or mobile property.
        :param content_type_uid:
        :return: ContentType
        """
        return ContentType(self.http_instance, content_type_uid)

    def asset(self, uid):
        """
        Assets refer to all the media files (images, videos, PDFs, audio files, and so on)
        uploaded in your Contentstack repository for future use.
        :param uid: asset_uid of the Asset
        :return: Asset

        -----------------------------
        Example: provide asset_uid to fetch single asset:
            >>> import contentstack
            >>> stack = Stack('api_key', 'delivery_token', 'environment')
            >>> asset_instance = stack.asset(uid='asset_uid')
            >>> asset = asset_instance.fetch()
        -----------------------------
        """
        if uid is None or not isinstance(uid, str):
            raise KeyError('Please provide a valid uid')
        return Asset(self.http_instance, uid=uid)

    def asset_query(self):
        """
        Assets refer to all the media files (images, videos, PDFs, audio files, and so on)
        uploaded in your Contentstack repository for future use.
        :return: Asset Instance
        -----------------------------
        Example: [All Assets]:
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> asset_query = stack.asset_query()
            >>> assets = asset_query.find()
        -----------------------------
        """
        return AssetQuery(self.http_instance)

    def sync_init(self, content_type_uid=None, from_date=None, locale=None, publish_type=None):
        """
        Constructs and initialises sync if no params provided else
        below mentioned params can be provided to get the response accordingly
        :param content_type_uid: subsequent syncs will only include the entries
                                 of the specified content_type.
        :param from_date: use from_date and specify the start date as its value.
        :param locale: specify the locale code as its value. However, if you do this,
                       the subsequent syncs will only include the entries of the specified locales.
        :param publish_type: Use the type parameter to get a specific
                             type of content. pass as follows:
        ``asset_published``, ``entry_published``,
        ``asset_unpublished``, ``asset_deleted``,
        ``entry_unpublished``, ``entry_deleted``, or ``content_type_deleted``.
        :return: list of sync items
        -------------------------------
        Example:
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> result = stack.sync_init(content_type_uid='content_type_uid',
                         from_date='date', locale='en-us', publish_type='asset_published')
        -------------------------------
        """
        self.sync_param['init'] = 'true'
        if content_type_uid is not None and isinstance(content_type_uid, str):
            self.sync_param['content_type_uid'] = content_type_uid
        if from_date is not None and isinstance(from_date, str):
            self.sync_param['from_date'] = from_date
        if locale is not None and isinstance(locale, str):
            self.sync_param['locale'] = locale
        if publish_type is not None and isinstance(publish_type, str):
            self.sync_param['publish_type'] = publish_type
        return self.__sync_request()

    def pagination(self, pagination_token: str):
        """
        If the result of the initial sync (or subsequent sync)
        contains more than 100 records, the response would be
        paginated. It provides pagination token in the response.
        However, you do not have to use the pagination token
        manually to get the next batch.
        :param pagination_token:
        :return: list of sync items
        ------------------------------
        Example:
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> result = stack.pagination('pagination_token')
        ------------------------------
        """
        if isinstance(pagination_token, str):
            self.sync_param = {'pagination_token': pagination_token}
        return self.__sync_request()

    def sync_token(self, sync_token):
        """You can use the sync token (that you receive after initial sync)
        to get the updated content next time. The sync token fetches
        only the content that was added
        after your last sync, and the details of the content that was deleted or updated.
        :param sync_token: sync_token
        :return: list of Sync Result
        ------------------------------
        [Example]:
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> result = stack.sync_token('sync_token')
        -------------------------------
        """
        if isinstance(sync_token, str):
            self.sync_param = {'sync_token': sync_token}
        return self.__sync_request()

    def __sync_request(self):
        r"""Sends a GET request.
        :param url URL for :class:`Request` object.
        in the query string for the :class:`Request`.
        :param \*\*kwargs: Optional arguments that ``request`` takes.
        :return: :class:`Response <Response>` object
        :rtype: requests.Response
        """
        base_url = '{}/stacks/sync'.format(self.http_instance.endpoint)
        self.sync_param['environment'] = self.http_instance.headers['environment']
        encoded_query = parse.urlencode(self.sync_param)
        url = '{}?{}'.format(base_url, encoded_query)
        result = self.http_instance.get(url)
        return result

    def image_transform(self, image_url, **kwargs):
        """
        This document is a detailed reference to Contentstack’s Image Delivery
        API and covers the parameters that you can add to the URL to retrieve,
        manipulate (or convert) image files and display
        it to your web or mobile properties.
        :param image_url: base url on which queries to apply
        :param kwargs: append queries to the asset URL.
        :return: instance of ImageTransform
        """
        if image_url is None or image_url == '':
            raise PermissionError('image_url required for the image_transformation')
        return ImageTransform(self.http_instance, image_url, **kwargs)



"""
EntryQueryable class contains common functions
that is used as parents class for the query and entry classes
"""


class EntryQueryable:
    """
    This class is base class for the Entry and Query class that shares common functions
    """

    def __init__(self):
        self.entry_queryable_param = {}

    def locale(self, locale: str):
        """
        Language code of which the entries needs to be included.
        Only the entries published in this locale will be displayed

        Arguments:
            locale {str} -- locale_code of the language of which the entries needs to be included.
            Only the entries published in this locale will be displayed

        :return: self: so you can chain this call.

        Example (locale for Entry):
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> entry = content_type.entry(uid='entry_uid')
            >>> entry.locale('en-us')
            >>> result = entry.fetch()

        Example (locale for Query):
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> query = content_type.query()
            >>> query.locale('en-us')
            >>> result = query.find()
        """
        self.entry_queryable_param['locale'] = locale
        return self

    def only(self, field_uid: str):
        """
        Specifies an array of only keys in BASE object that would be included in the response.
        It refers to the top-level fields of the schema
        :param field_uid: Array of the only reference keys to be included in response
        Returns:
            self -- so you can chain this call.
        """
        if field_uid is not None:
            if isinstance(field_uid, str):
                self.entry_queryable_param['only[BASE][]'] = field_uid
            else:
                raise KeyError("Invalid field_uid provided")
        return self

    def excepts(self, field_uid: str):
        """
        Specifies list of field_uid that would be excluded from the response.
        It refers to the top-level fields of the schema
        :param field_uid: to be excluded from the response.
        :return: self -- so you can chain this call.
        """
        if field_uid is not None:
            if isinstance(field_uid, str):
                self.entry_queryable_param['except[BASE][]'] = field_uid
            else:
                raise KeyError("Invalid field_uid provided")
        return self

    def include_reference(self, field_uid):
        """
        **Include Reference:**
        When you fetch an entry of a content type that has a reference field,
        by default, the content of the referred entry is not fetched.
        It only fetches the UID of the referred entry, along with the content of
        the specified entry.

        Note: The maximum reference depth limit to which a multiple content type
        referencing Reference field works is three levels deep

        Arguments:
        Array of the only reference keys to be included in response
        field_uid {str or list of str} -- [str/list of str] of field_uid on
        which include operation to perform

        Returns:
            self -- So you can chain this call.

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> entry = stack.content_type('content_type')
            >>> entry("entry_uid").include_reference(["categories", "brand"])
            >>> result = entry.fetch()
        """
        if field_uid is not None and isinstance(field_uid, (str, list)):
            self.entry_queryable_param["include[]"] = field_uid
        return self

    def include_content_type(self):
        """
        This method also includes the ContentType in the entry
        :return: self: so you can chain this call.
        -------------------------------
        [Example: for Entry]
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> entry = content_type.entry('uid')
            >>> entry.include_content_type()
            >>> result = entry.fetch()

        -------------------------------

        [Example: for Query:]

            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> query = content_type.query()
            >>> query.include_content_type()
            >>> result = query.find()
        -------------------------------
        """
        self.entry_queryable_param['include_content_type'] = 'true'
        self.entry_queryable_param['include_global_field_schema'] = 'true'
        return self

    def include_reference_content_type_uid(self):
        """
        This method also includes the content type UIDs
        of the referenced entries returned in the response
        Returns:
            :return: self: so you can chain this call.

        [Example for Query]
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> content_type = stack.content_type('content_type_uid')
            >>> query = content_type.query()
            >>> query = query.include_reference_content_type_uid()
            >>> result = query.find()

        [Example for Entry]
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> entry = stack.content_type('content_type_uid').entry('entry_uid')
            >>> entry = entry.include_reference_content_type_uid()
            >>> result = entry.fetch()
        """
        self.entry_queryable_param['include_reference_content_type_uid'] = 'true'
        return self

    def add_param(self, key: str, value: str):
        """
        This method adds key and value to an Entry.
        :param key: The key as string which needs to be added to an Entry
        :param value: The value as string which needs to be added to an Entry
        :return: self: object, so you can chain this call.

        Example: Call from Query =>
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> query = stack.content_type('content_type_uid').query()
            >>> query = query.include_reference_content_type_uid()
            >>> result = query.find()

        Example: Call from Entry =>
            >>> import contentstack
            >>> stack = contentstack.Stack('api_key', 'delivery_token', 'environment')
            >>> entry = stack.content_type('content_type_uid').entry('entry_uid')
            >>> entry = entry.include_reference_content_type_uid()
            >>> result = entry.fetch()

        """
        if None not in (key, value):
            self.entry_queryable_param[key] = value



# https://packaging.python.org/tutorials/packaging-projects/#creating-setup-py
# setup.py is the build script for setuptools.
# It tells setuptools about your package (such
# as the name and version) as well as which code files to include

import os

import contentstack

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    long_description = readme.read()

# from os import path
# this_directory = path.abspath(path.dirname(__file__))
# with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
#     long_description = f.read()

setup(
    title="contentstack-python",
    name="Contentstack",
    status="Active",
    type="process",
    created="17 Jun",
    keywords=contentstack.__title__,
    version=contentstack.__version__,
    author="Contentstack",
    author_email="shailesh.mishra@contentstack.com",
    description="Contentstack is a headless CMS with an API-first approach.",
    # long_description="Contentstack is a headless CMS with an API-first approach. It is a CMS that developers can use "
    #                  "to build powerful cross-platform applications in their favorite languages. Build your "
    #                  "application frontend, and Contentstack will take care of the rest",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/contentstack/contentstack-python",
    packages=['contentstack'],
    license='MIT',
    test_suite='tests.all_tests',
    install_requires=['requests>=1.1.0'],
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    python_requires='>=3.6',
    zip_safe=False,
)



"""
The __init__.py files are required to make Python treat the directories as containing
packages; this is done to prevent directories with a common name, such as string,
from unintentionally hiding valid modules that occur later on the module search path

Used: Safety checks your installed dependencies for known security vulnerabilities
file __init__.py contains package information like
__author__, __status__, __version__, __endpoint__ and __email__

"""
from .entry import Entry
from .asset import Asset
from .contenttype import ContentType
from .https_connection import HTTPSConnection
from .stack import Stack
from .utility import Utils
# from contentstack import *

__title__ = 'contentstack-python'
__author__ = 'Contentstack'
__status__ = 'debug'
__version__ = '1.1.0'
__endpoint__ = 'cdn.contentstack.io'
__email__ = 'shailesh.mishra@contentstack.com'
